/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_GeneratedAtomList_h__
#define mozilla_dom_GeneratedAtomList_h__

namespace mozilla {
namespace dom {

struct AnimationEventInitAtoms
{
  jsid animationName_id;
  jsid elapsedTime_id;
  jsid pseudoElement_id;
};

struct AppInfoAtoms
{
  jsid isCoreApp_id;
  jsid path_id;
};

struct AppNotificationServiceOptionsAtoms
{
  jsid dir_id;
  jsid id_id;
  jsid lang_id;
  jsid manifestURL_id;
  jsid textClickable_id;
};

struct ArchiveReaderOptionsAtoms
{
  jsid encoding_id;
};

struct AsyncScrollEventDetailAtoms
{
  jsid height_id;
  jsid left_id;
  jsid scrollHeight_id;
  jsid scrollWidth_id;
  jsid top_id;
  jsid width_id;
};

struct BlobEventInitAtoms
{
  jsid data_id;
};

struct BlobPropertyBagAtoms
{
  jsid endings_id;
  jsid type_id;
};

struct CallEventInitAtoms
{
  jsid call_id;
};

struct CallGroupErrorEventInitAtoms
{
  jsid message_id;
  jsid name_id;
};

struct CameraPictureOptionsAtoms
{
  jsid dateTime_id;
  jsid fileFormat_id;
  jsid pictureSize_id;
  jsid position_id;
  jsid rotation_id;
};

struct CameraSelectorAtoms
{
  jsid camera_id;
};

struct ClipboardEventInitAtoms
{
  jsid data_id;
  jsid dataType_id;
};

struct CloseEventInitAtoms
{
  jsid code_id;
  jsid reason_id;
  jsid wasClean_id;
};

struct CompositionClauseParametersAtoms
{
  jsid length_id;
  jsid selectionType_id;
};

struct ConnStatusDictAtoms
{
  jsid status_id;
};

struct ContactAddressInitAtoms
{
  jsid countryName_id;
  jsid locality_id;
  jsid postalCode_id;
  jsid pref_id;
  jsid region_id;
  jsid streetAddress_id;
  jsid type_id;
};

struct ContactFieldInitAtoms
{
  jsid pref_id;
  jsid type_id;
  jsid value_id;
};

struct ContactFindOptionsAtoms
{
  jsid filterBy_id;
  jsid filterLimit_id;
  jsid filterOp_id;
  jsid filterValue_id;
};

struct ContactFindSortOptionsAtoms
{
  jsid sortBy_id;
  jsid sortOrder_id;
};

struct ContactPropertiesAtoms
{
  jsid additionalName_id;
  jsid adr_id;
  jsid anniversary_id;
  jsid bday_id;
  jsid category_id;
  jsid email_id;
  jsid familyName_id;
  jsid genderIdentity_id;
  jsid givenName_id;
  jsid honorificPrefix_id;
  jsid honorificSuffix_id;
  jsid impp_id;
  jsid jobTitle_id;
  jsid key_id;
  jsid name_id;
  jsid nickname_id;
  jsid note_id;
  jsid org_id;
  jsid photo_id;
  jsid sex_id;
  jsid tel_id;
  jsid url_id;
};

struct ContactTelFieldInitAtoms
{
  jsid carrier_id;
};

struct CustomEventInitAtoms
{
  jsid detail_id;
};

struct DNSCacheDictAtoms
{
  jsid entries_id;
};

struct DNSLookupDictAtoms
{
  jsid address_id;
  jsid answer_id;
  jsid error_id;
};

struct DOMTransactionEventInitAtoms
{
  jsid transactions_id;
};

struct DOMWindowResizeEventDetailAtoms
{
  jsid height_id;
  jsid width_id;
};

struct DataStoreChangeEventInitAtoms
{
  jsid id_id;
  jsid operation_id;
  jsid revisionId_id;
};

struct DataStoreTaskAtoms
{
  jsid data_id;
  jsid id_id;
  jsid operation_id;
  jsid revisionId_id;
};

struct DeviceLightEventInitAtoms
{
  jsid value_id;
};

struct DeviceOrientationEventInitAtoms
{
  jsid absolute_id;
  jsid alpha_id;
  jsid beta_id;
  jsid gamma_id;
};

struct DeviceProximityEventInitAtoms
{
  jsid max_id;
  jsid min_id;
  jsid value_id;
};

struct DeviceStorageChangeEventInitAtoms
{
  jsid path_id;
  jsid reason_id;
};

struct DeviceStorageEnumerationParametersAtoms
{
  jsid since_id;
};

struct DnsCacheEntryAtoms
{
  jsid expiration_id;
  jsid family_id;
  jsid hostaddr_id;
  jsid hostname_id;
};

struct DownloadEventInitAtoms
{
  jsid download_id;
};

struct ElementRegistrationOptionsAtoms
{
  jsid lifecycle_id;
  jsid prototype_id;
};

struct ElementReplaceEventInitAtoms
{
  jsid upgrade_id;
};

struct ErrorEventInitAtoms
{
  jsid column_id;
  jsid filename_id;
  jsid lineno_id;
  jsid message_id;
};

struct EventInitAtoms
{
  jsid bubbles_id;
  jsid cancelable_id;
};

struct EventSourceInitAtoms
{
  jsid withCredentials_id;
};

struct FilePropertyBagAtoms
{
  jsid name_id;
};

struct FocusEventInitAtoms
{
  jsid relatedTarget_id;
};

struct GamepadAxisMoveEventInitAtoms
{
  jsid axis_id;
  jsid value_id;
};

struct GamepadButtonEventInitAtoms
{
  jsid button_id;
};

struct GamepadEventInitAtoms
{
  jsid gamepad_id;
};

struct GetNotificationOptionsAtoms
{
  jsid tag_id;
};

struct HalfOpenInfoDictAtoms
{
  jsid speculative_id;
};

struct HashChangeEventInitAtoms
{
  jsid newURL_id;
  jsid oldURL_id;
};

struct HttpConnDictAtoms
{
  jsid connections_id;
};

struct HttpConnInfoAtoms
{
  jsid protocolVersion_id;
  jsid rtt_id;
  jsid ttl_id;
};

struct HttpConnectionElementAtoms
{
  jsid active_id;
  jsid halfOpens_id;
  jsid host_id;
  jsid idle_id;
  jsid port_id;
  jsid spdy_id;
  jsid ssl_id;
};

struct IDBIndexParametersAtoms
{
  jsid multiEntry_id;
  jsid unique_id;
};

struct IDBObjectStoreParametersAtoms
{
  jsid autoIncrement_id;
  jsid keyPath_id;
};

struct IDBOpenDBOptionsAtoms
{
  jsid storage_id;
  jsid version_id;
};

struct IDBVersionChangeEventInitAtoms
{
  jsid newVersion_id;
  jsid oldVersion_id;
};

struct IccChangeEventInitAtoms
{
  jsid iccId_id;
};

struct InspectorRGBTripleAtoms
{
  jsid b_id;
  jsid g_id;
  jsid r_id;
};

struct LifecycleCallbacksAtoms
{
  jsid created_id;
};

struct MediaConstraintSetAtoms
{
  jsid MozDontOfferDataChannel_id;
  jsid OfferToReceiveAudio_id;
  jsid OfferToReceiveVideo_id;
};

struct MediaConstraintsAtoms
{
  jsid mandatory_id;
  jsid optional_id;
};

struct MediaConstraintsInternalAtoms
{
  jsid mandatory_id;
  jsid optional_id;
};

struct MediaStreamConstraintsAtoms
{
  jsid audio_id;
  jsid fake_id;
  jsid picture_id;
  jsid video_id;
};

struct MediaStreamConstraintsInternalAtoms
{
  jsid audio_id;
  jsid audiom_id;
  jsid fake_id;
  jsid picture_id;
  jsid video_id;
  jsid videom_id;
};

struct MediaStreamEventInitAtoms
{
  jsid stream_id;
};

struct MediaTrackConstraintSetAtoms
{
  jsid facingMode_id;
};

struct MediaTrackConstraintsAtoms
{
  jsid mandatory_id;
  jsid optional_id;
};

struct MediaTrackConstraintsInternalAtoms
{
  jsid mandatory_id;
  jsid optional_id;
};

struct MessageEventInitAtoms
{
  jsid data_id;
  jsid lastEventId_id;
  jsid origin_id;
  jsid ports_id;
  jsid source_id;
};

struct MmsAttachmentAtoms
{
  jsid content_id;
  jsid id_id;
  jsid location_id;
};

struct MmsParametersAtoms
{
  jsid attachments_id;
  jsid receivers_id;
  jsid smil_id;
  jsid subject_id;
};

struct MmsSendParametersAtoms
{
  jsid serviceId_id;
};

struct MouseEventInitAtoms
{
  jsid altKey_id;
  jsid bubbles_id;
  jsid button_id;
  jsid buttons_id;
  jsid cancelable_id;
  jsid clientX_id;
  jsid clientY_id;
  jsid ctrlKey_id;
  jsid detail_id;
  jsid metaKey_id;
  jsid relatedTarget_id;
  jsid screenX_id;
  jsid screenY_id;
  jsid shiftKey_id;
  jsid view_id;
};

struct MozApplicationEventInitAtoms
{
  jsid application_id;
};

struct MozContactChangeEventInitAtoms
{
  jsid contactID_id;
  jsid reason_id;
};

struct MozInterAppMessageEventInitAtoms
{
  jsid data_id;
};

struct MozMmsEventInitAtoms
{
  jsid message_id;
};

struct MozSettingsEventInitAtoms
{
  jsid settingName_id;
  jsid settingValue_id;
};

struct MozSmsEventInitAtoms
{
  jsid message_id;
};

struct MozStkCommandEventInitAtoms
{
  jsid command_id;
};

struct MozXMLHttpRequestParametersAtoms
{
  jsid mozAnon_id;
  jsid mozSystem_id;
};

struct MutationObserverInitAtoms
{
  jsid attributeFilter_id;
  jsid attributeOldValue_id;
  jsid attributes_id;
  jsid characterData_id;
  jsid characterDataOldValue_id;
  jsid childList_id;
  jsid subtree_id;
};

struct MutationObservingInfoAtoms
{
  jsid observedNode_id;
};

struct NotificationOptionsAtoms
{
  jsid body_id;
  jsid dir_id;
  jsid icon_id;
  jsid lang_id;
  jsid tag_id;
};

struct OpenWindowEventDetailAtoms
{
  jsid features_id;
  jsid frameElement_id;
  jsid name_id;
  jsid url_id;
};

struct PageTransitionEventInitAtoms
{
  jsid persisted_id;
};

struct PointerEventInitAtoms
{
  jsid height_id;
  jsid isPrimary_id;
  jsid pointerId_id;
  jsid pointerType_id;
  jsid pressure_id;
  jsid tiltX_id;
  jsid tiltY_id;
  jsid width_id;
};

struct PopStateEventInitAtoms
{
  jsid state_id;
};

struct PopupBlockedEventInitAtoms
{
  jsid popupWindowFeatures_id;
  jsid popupWindowName_id;
  jsid popupWindowURI_id;
  jsid requestingWindow_id;
};

struct PositionOptionsAtoms
{
  jsid enableHighAccuracy_id;
  jsid maximumAge_id;
  jsid timeout_id;
};

struct ProgressEventInitAtoms
{
  jsid lengthComputable_id;
  jsid loaded_id;
  jsid total_id;
};

struct RTCCodecStatsAtoms
{
  jsid channels_id;
  jsid clockRate_id;
  jsid codec_id;
  jsid parameters_id;
  jsid payloadType_id;
};

struct RTCConfigurationAtoms
{
  jsid iceServers_id;
};

struct RTCDataChannelEventInitAtoms
{
  jsid channel_id;
};

struct RTCDataChannelInitAtoms
{
  jsid id_id;
  jsid maxRetransmitNum_id;
  jsid maxRetransmitTime_id;
  jsid maxRetransmits_id;
  jsid negotiated_id;
  jsid ordered_id;
  jsid outOfOrderAllowed_id;
  jsid preset_id;
  jsid protocol_id;
  jsid stream_id;
};

struct RTCIceCandidateInitAtoms
{
  jsid candidate_id;
  jsid sdpMLineIndex_id;
  jsid sdpMid_id;
};

struct RTCIceCandidatePairStatsAtoms
{
  jsid componentId_id;
  jsid localCandidateId_id;
  jsid mozPriority_id;
  jsid nominated_id;
  jsid readable_id;
  jsid remoteCandidateId_id;
  jsid selected_id;
  jsid state_id;
};

struct RTCIceCandidateStatsAtoms
{
  jsid candidateId_id;
  jsid candidateType_id;
  jsid ipAddress_id;
  jsid portNumber_id;
};

struct RTCIceComponentStatsAtoms
{
  jsid activeConnection_id;
  jsid bytesReceived_id;
  jsid bytesSent_id;
  jsid component_id;
  jsid transportId_id;
};

struct RTCIceServerAtoms
{
  jsid credential_id;
  jsid url_id;
  jsid username_id;
};

struct RTCInboundRTPStreamStatsAtoms
{
  jsid bytesReceived_id;
  jsid jitter_id;
  jsid packetsReceived_id;
};

struct RTCMediaStreamStatsAtoms
{
  jsid streamIdentifier_id;
  jsid trackIds_id;
};

struct RTCMediaStreamTrackStatsAtoms
{
  jsid audioLevel_id;
  jsid first_id;
  jsid frameHeight_id;
  jsid frameWidth_id;
  jsid framesDecoded_id;
  jsid framesPerSecond_id;
  jsid framesReceived_id;
  jsid framesSent_id;
  jsid remoteSource_id;
  jsid ssrcIds_id;
  jsid trackIdentifier_id;
};

struct RTCOutboundRTPStreamStatsAtoms
{
  jsid bytesSent_id;
  jsid packetsSent_id;
};

struct RTCPeerConnectionIceEventInitAtoms
{
  jsid candidate_id;
};

struct RTCRTPStreamStatsAtoms
{
  jsid codecId_id;
  jsid isRemote_id;
  jsid mediaTrackId_id;
  jsid remoteId_id;
  jsid ssrc_id;
  jsid transportId_id;
};

struct RTCSessionDescriptionInitAtoms
{
  jsid sdp_id;
  jsid type_id;
};

struct RTCStatsAtoms
{
  jsid id_id;
  jsid timestamp_id;
  jsid type_id;
};

struct RTCStatsReportInternalAtoms
{
  jsid codecStats_id;
  jsid iceCandidatePairStats_id;
  jsid iceCandidateStats_id;
  jsid iceComponentStats_id;
  jsid inboundRTPStreamStats_id;
  jsid mediaStreamStats_id;
  jsid mediaStreamTrackStats_id;
  jsid outboundRTPStreamStats_id;
  jsid pcid_id;
  jsid rtpStreamStats_id;
  jsid transportStats_id;
};

struct RTCTransportStatsAtoms
{
  jsid bytesReceived_id;
  jsid bytesSent_id;
};

struct RecordErrorEventInitAtoms
{
  jsid name_id;
};

struct ScrollOptionsAtoms
{
  jsid behavior_id;
};

struct SettingChangeAtoms
{
  jsid settingName_id;
  jsid settingValue_id;
};

struct SmartCardEventInitAtoms
{
  jsid tokenName_id;
};

struct SmsSendParametersAtoms
{
  jsid serviceId_id;
};

struct SocketElementAtoms
{
  jsid active_id;
  jsid host_id;
  jsid port_id;
  jsid received_id;
  jsid sent_id;
  jsid tcp_id;
};

struct SocketsDictAtoms
{
  jsid received_id;
  jsid sent_id;
  jsid sockets_id;
};

struct SpeechRecognitionErrorInitAtoms
{
  jsid error_id;
  jsid message_id;
};

struct SpeechRecognitionEventInitAtoms
{
  jsid emma_id;
  jsid interpretation_id;
  jsid resultIndex_id;
  jsid results_id;
};

struct SpeechSynthesisEventInitAtoms
{
  jsid charIndex_id;
  jsid elapsedTime_id;
  jsid name_id;
};

struct StorageEventInitAtoms
{
  jsid key_id;
  jsid newValue_id;
  jsid oldValue_id;
  jsid storageArea_id;
  jsid url_id;
};

struct StyleRuleChangeEventInitAtoms
{
  jsid rule_id;
  jsid stylesheet_id;
};

struct StyleSheetApplicableStateChangeEventInitAtoms
{
  jsid applicable_id;
  jsid stylesheet_id;
};

struct StyleSheetChangeEventInitAtoms
{
  jsid documentSheet_id;
  jsid stylesheet_id;
};

struct TextDecodeOptionsAtoms
{
  jsid stream_id;
};

struct TextDecoderOptionsAtoms
{
  jsid fatal_id;
};

struct TextEncodeOptionsAtoms
{
  jsid stream_id;
};

struct TrackEventInitAtoms
{
  jsid track_id;
};

struct TransitionEventInitAtoms
{
  jsid elapsedTime_id;
  jsid propertyName_id;
  jsid pseudoElement_id;
};

struct UIEventInitAtoms
{
  jsid detail_id;
  jsid view_id;
};

struct UserProximityEventInitAtoms
{
  jsid near_id;
};

struct WebGLContextAttributesAtoms
{
  jsid alpha_id;
  jsid antialias_id;
  jsid depth_id;
  jsid premultipliedAlpha_id;
  jsid preserveDrawingBuffer_id;
  jsid stencil_id;
};

struct WebSocketDictAtoms
{
  jsid websockets_id;
};

struct WebSocketElementAtoms
{
  jsid encrypted_id;
  jsid hostport_id;
  jsid msgreceived_id;
  jsid msgsent_id;
  jsid receivedsize_id;
  jsid sentsize_id;
};

struct WheelEventInitAtoms
{
  jsid deltaMode_id;
  jsid deltaX_id;
  jsid deltaY_id;
  jsid deltaZ_id;
};

struct WifiCommandOptionsAtoms
{
  jsid cmd_id;
  jsid defaultValue_id;
  jsid dns1_id;
  jsid dns2_id;
  jsid gateway_id;
  jsid id_id;
  jsid ifname_id;
  jsid ipaddr_id;
  jsid key_id;
  jsid mask_id;
  jsid request_id;
  jsid route_id;
  jsid value_id;
};

struct WifiResultOptionsAtoms
{
  jsid broadcast_str_id;
  jsid dns1_id;
  jsid dns1_str_id;
  jsid dns2_id;
  jsid dns2_str_id;
  jsid error_id;
  jsid gateway_id;
  jsid gateway_str_id;
  jsid id_id;
  jsid ipaddr_id;
  jsid ipaddr_str_id;
  jsid lease_id;
  jsid mask_id;
  jsid mask_str_id;
  jsid reply_id;
  jsid route_id;
  jsid server_id;
  jsid server_str_id;
  jsid status_id;
  jsid value_id;
  jsid vendor_str_id;
};

struct PerThreadAtomCache : public AnimationEventInitAtoms,
                            public AppInfoAtoms,
                            public AppNotificationServiceOptionsAtoms,
                            public ArchiveReaderOptionsAtoms,
                            public AsyncScrollEventDetailAtoms,
                            public BlobEventInitAtoms,
                            public BlobPropertyBagAtoms,
                            public CallEventInitAtoms,
                            public CallGroupErrorEventInitAtoms,
                            public CameraPictureOptionsAtoms,
                            public CameraSelectorAtoms,
                            public ClipboardEventInitAtoms,
                            public CloseEventInitAtoms,
                            public CompositionClauseParametersAtoms,
                            public ConnStatusDictAtoms,
                            public ContactAddressInitAtoms,
                            public ContactFieldInitAtoms,
                            public ContactFindOptionsAtoms,
                            public ContactFindSortOptionsAtoms,
                            public ContactPropertiesAtoms,
                            public ContactTelFieldInitAtoms,
                            public CustomEventInitAtoms,
                            public DNSCacheDictAtoms,
                            public DNSLookupDictAtoms,
                            public DOMTransactionEventInitAtoms,
                            public DOMWindowResizeEventDetailAtoms,
                            public DataStoreChangeEventInitAtoms,
                            public DataStoreTaskAtoms,
                            public DeviceLightEventInitAtoms,
                            public DeviceOrientationEventInitAtoms,
                            public DeviceProximityEventInitAtoms,
                            public DeviceStorageChangeEventInitAtoms,
                            public DeviceStorageEnumerationParametersAtoms,
                            public DnsCacheEntryAtoms,
                            public DownloadEventInitAtoms,
                            public ElementRegistrationOptionsAtoms,
                            public ElementReplaceEventInitAtoms,
                            public ErrorEventInitAtoms,
                            public EventInitAtoms,
                            public EventSourceInitAtoms,
                            public FilePropertyBagAtoms,
                            public FocusEventInitAtoms,
                            public GamepadAxisMoveEventInitAtoms,
                            public GamepadButtonEventInitAtoms,
                            public GamepadEventInitAtoms,
                            public GetNotificationOptionsAtoms,
                            public HalfOpenInfoDictAtoms,
                            public HashChangeEventInitAtoms,
                            public HttpConnDictAtoms,
                            public HttpConnInfoAtoms,
                            public HttpConnectionElementAtoms,
                            public IDBIndexParametersAtoms,
                            public IDBObjectStoreParametersAtoms,
                            public IDBOpenDBOptionsAtoms,
                            public IDBVersionChangeEventInitAtoms,
                            public IccChangeEventInitAtoms,
                            public InspectorRGBTripleAtoms,
                            public LifecycleCallbacksAtoms,
                            public MediaConstraintSetAtoms,
                            public MediaConstraintsAtoms,
                            public MediaConstraintsInternalAtoms,
                            public MediaStreamConstraintsAtoms,
                            public MediaStreamConstraintsInternalAtoms,
                            public MediaStreamEventInitAtoms,
                            public MediaTrackConstraintSetAtoms,
                            public MediaTrackConstraintsAtoms,
                            public MediaTrackConstraintsInternalAtoms,
                            public MessageEventInitAtoms,
                            public MmsAttachmentAtoms,
                            public MmsParametersAtoms,
                            public MmsSendParametersAtoms,
                            public MouseEventInitAtoms,
                            public MozApplicationEventInitAtoms,
                            public MozContactChangeEventInitAtoms,
                            public MozInterAppMessageEventInitAtoms,
                            public MozMmsEventInitAtoms,
                            public MozSettingsEventInitAtoms,
                            public MozSmsEventInitAtoms,
                            public MozStkCommandEventInitAtoms,
                            public MozXMLHttpRequestParametersAtoms,
                            public MutationObserverInitAtoms,
                            public MutationObservingInfoAtoms,
                            public NotificationOptionsAtoms,
                            public OpenWindowEventDetailAtoms,
                            public PageTransitionEventInitAtoms,
                            public PointerEventInitAtoms,
                            public PopStateEventInitAtoms,
                            public PopupBlockedEventInitAtoms,
                            public PositionOptionsAtoms,
                            public ProgressEventInitAtoms,
                            public RTCCodecStatsAtoms,
                            public RTCConfigurationAtoms,
                            public RTCDataChannelEventInitAtoms,
                            public RTCDataChannelInitAtoms,
                            public RTCIceCandidateInitAtoms,
                            public RTCIceCandidatePairStatsAtoms,
                            public RTCIceCandidateStatsAtoms,
                            public RTCIceComponentStatsAtoms,
                            public RTCIceServerAtoms,
                            public RTCInboundRTPStreamStatsAtoms,
                            public RTCMediaStreamStatsAtoms,
                            public RTCMediaStreamTrackStatsAtoms,
                            public RTCOutboundRTPStreamStatsAtoms,
                            public RTCPeerConnectionIceEventInitAtoms,
                            public RTCRTPStreamStatsAtoms,
                            public RTCSessionDescriptionInitAtoms,
                            public RTCStatsAtoms,
                            public RTCStatsReportInternalAtoms,
                            public RTCTransportStatsAtoms,
                            public RecordErrorEventInitAtoms,
                            public ScrollOptionsAtoms,
                            public SettingChangeAtoms,
                            public SmartCardEventInitAtoms,
                            public SmsSendParametersAtoms,
                            public SocketElementAtoms,
                            public SocketsDictAtoms,
                            public SpeechRecognitionErrorInitAtoms,
                            public SpeechRecognitionEventInitAtoms,
                            public SpeechSynthesisEventInitAtoms,
                            public StorageEventInitAtoms,
                            public StyleRuleChangeEventInitAtoms,
                            public StyleSheetApplicableStateChangeEventInitAtoms,
                            public StyleSheetChangeEventInitAtoms,
                            public TextDecodeOptionsAtoms,
                            public TextDecoderOptionsAtoms,
                            public TextEncodeOptionsAtoms,
                            public TrackEventInitAtoms,
                            public TransitionEventInitAtoms,
                            public UIEventInitAtoms,
                            public UserProximityEventInitAtoms,
                            public WebGLContextAttributesAtoms,
                            public WebSocketDictAtoms,
                            public WebSocketElementAtoms,
                            public WheelEventInitAtoms,
                            public WifiCommandOptionsAtoms,
                            public WifiResultOptionsAtoms
{
};

} // namespace dom
} // namespace mozilla


#endif // mozilla_dom_GeneratedAtomList_h__
